import fs from 'fs'
import { Server } from 'socket.io'
import { TSocket } from '../../../../utilities/types'
import prisma from '../../../../utilities/prisma'
import characterRepository from '../../../../repositories/characterRepository'
import { getPublicPath } from '../../../../utilities/storage'
import { gameMasterLogger } from '../../../../utilities/logger'

interface IPayload {
  object: string
}

export default class CharacterTypeDeleteEvent {
  constructor(
    private readonly io: Server,
    private readonly socket: TSocket
  ) {}

  public listen(): void {
    this.socket.on('gm:characterType:remove', this.handleEvent.bind(this))
  }

  private async handleEvent(data: IPayload, callback: (response: boolean) => void): Promise<void> {
    const character = await characterRepository.getById(this.socket.characterId as number)
    if (!character) return callback(false)

    if (character.role !== 'gm') {
      return callback(false)
    }

    try {
      await prisma.object.delete({
        where: {
          id: data.object
        }
      })

      // remove the tile from the disk
      const finalFilePath = getPublicPath('objects', data.object + '.png')
      fs.unlink(finalFilePath, (err) => {
        if (err) {
          gameMasterLogger.error(`Error deleting object ${data.object}: ${err.message}`)
          callback(false)
          return
        }

        callback(true)
      })
    } catch (error) {
      gameMasterLogger.error(`Error deleting object ${data.object}: ${error instanceof Error ? error.message : String(error)}`)
      callback(false)
    }
  }
}
